SystemOrganization addCategory: #GODBases!

Object subclass: #GDBClientController
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GODBases'!

!GDBClientController commentStamp: 'lacm 11/15/2014 21:07' prior: 0!
A GDBClientController is a main class that permits differents transactions in our database.

Before following these instructions, first set up your server in a separate running image by following the instructions in the GDBServerController class. After you have the server running, switch back to this image and do the following.
	
1.-  First, to conect a client with the database run the following:	

			GDBClientController initializeSession.

2.- To create the database, run:

			GDBClientController createDatabase.
				
	If you want to create only the database for a GODData, run:
	
			GDBClientController createGODDataCollection.
			
	If you want to create only the database for a GCConference, run:		
			
			GDBClientController createConferenceCollection.
			
3.  To reset the database, run the following:	 
	
			GDBClientController resetAllDatabase.
	
	If you want to reset only the database for a GODData, run:
	
			GDBClientController resetData			
	
	If you want to reset only the database for a GCConference, run:		
								
			GDBClientController resetConferenceData
	
4. Transactions for a GODData object.

			GDBClientController add:  myGODData.
			GDBClientController remove:  myGODData.			
			GDBClientController update:  myOldGODData with: myNewGODData.						
			GDBClientController searchAuthor: 'anAuthorName'.	
			GDBClientController searchTitle: 'aTitleName'.	
			GDBClientController search	For: 'aBlock'.

5. Transactions for a GCConference object.

			GDBClientController saveConferences:  myConferenceCollection.
			GDBClientController loadConferences.
			
6. To update any change by other client, before any transaction run the following:
	
			GDBClientController refresh.
			
7. To change a client session, run the following:

			GDBClientController refresh: 'nameOfOtherClient'.			

8. To disconnect the client session, run the following:

			GDBClientController release.!

!GDBClientController class methodsFor: 'adding' stamp: 'lacm 11/18/2014 21:12'!
add: aGODData
	"This method permits to insert a GODData object in the database"
      |myReader|
      (aGODData class = GODData) ifFalse: [GDBDataMustBeAnGODData signal].
      (aGODData author isNil) ifTrue:[GODDataAuthorMustContainsSomeInformation signal].
      (aGODData title isNil) ifTrue:[GODDataTitleMustContainsSomeInformation signal].
	((self godDataExists: aGODData) = True)
		ifTrue:[GODDataAlreadyExistsInTheDatabase signal].
      GDBMagmaClient commit: [self database add:  (self updateId: aGODData).].	
	Transcript show: (aGODData,' object was added!!!!'); cr; yourself.
	myReader := self database where:[:reader| reader read: #id from: 1]  distinct: true sortBy: #id descending: true.		
	myReader do: [:each| Transcript show: 'id: ',each id; show: '  author: '; show: each author; show: '  title: '; show: each title; cr; yourself.].
! !

!GDBClientController class methodsFor: 'private' stamp: 'lacm 11/19/2014 19:57'!
conferenceCollectionExists
	|auxDatabaseConference|
	auxDatabaseConference := GDBMagmaClient root keys.
	auxDatabaseConference do: [:e| (e ='gdbconference') ifTrue:[^True]; ifFalse:[^False]]
	
	
	! !

!GDBClientController class methodsFor: 'creating' stamp: 'lacm 11/15/2014 20:37'!
createConferenceCollection
"This method creates a collection to store a collection of GCConference objects in the database"
	self initializeSession.
	GDBMagmaClient commit: [GDBMagmaClient root at: 'gdbconference' put: OrderedCollection new. ].
! !

!GDBClientController class methodsFor: 'creating' stamp: 'lacm 11/15/2014 20:37'!
createDatabase
"This method creates two collections to store GODData objects and a collection of GCConference objects in the database"
"	self initializeSession."
	GDBMagmaClient commit: [
								self createGODDataCollection.
								self createConferenceCollection.].
	GDBMagmaClient root.
! !

!GDBClientController class methodsFor: 'creating' stamp: 'lacm 11/15/2014 20:37'!
createGODDataCollection
"This method creates a collection to store GODData objects in the database"
"	self initializeSession."
	GDBMagmaClient commit: [
								GDBMagmaClient root at: 'gdbdata' put: (MagmaCollection new		
													addIndex:(MaIntegerIndex attribute: #id);	
													addIndex:(MaSearchStringIndex attribute: #author);		
													addIndex:(MaSearchStringIndex attribute: #title);		
													yourself).
								].
! !

!GDBClientController class methodsFor: 'private' stamp: 'lacm 11/12/2014 11:45'!
database
      "new key for the root dictionary"
	^ GDBMagmaClient root at: 'gdbdata'.! !

!GDBClientController class methodsFor: 'private' stamp: 'lacm 11/14/2014 14:48'!
databaseConference
      "new key for the root dictionary"	
	^ GDBMagmaClient root at: 'gdbconference'.! !

!GDBClientController class methodsFor: 'getting' stamp: 'lacm 11/14/2014 16:50'!
getAll   
	"This method gets all element of the data base."
	| myReader|	
	myReader := self database.
	^myReader! !

!GDBClientController class methodsFor: 'private' stamp: 'lacm 11/19/2014 19:57'!
godDataCollectionExists
	|auxGODDataCollection|
	auxGODDataCollection := GDBMagmaClient root keys.
	auxGODDataCollection do: [:e| (e ='gdbdata') ifTrue:[^True]; ifFalse:[^False]]
	
	
	! !

!GDBClientController class methodsFor: 'adding' stamp: 'lacm 11/19/2014 21:29'!
godDataExists: aGODData
	"This method permits to verify if a GODData object exists in the database"
      |myReader|
	(self database size = 0)
	ifFalse:[ myReader:= self getAll."database where:[:reader | reader read:#id from:1] distinct:true sortBy:#id descending:true."
			myReader do:[:each| (each=aGODData)
									ifTrue:[^True]
						   ].
		    ].
	^False! !

!GDBClientController class methodsFor: 'accessing' stamp: 'lacm 11/14/2014 14:10'!
initializeSession
	^GDBMagmaClient session! !

!GDBClientController class methodsFor: 'getting' stamp: 'lacm 11/19/2014 19:45'!
loadConferences
	(self conferenceCollectionExists = True) 
		ifTrue: [ (self databaseConference size > 0) 
					ifTrue:[^self databaseConference].
				].
	^nil! !

!GDBClientController class methodsFor: 'connection' stamp: 'lacm 11/14/2014 16:51'!
refresh
	"This method permits to refresh a client session"
	|aclient|
	aclient:=GDBMagmaClient aClientName. 
	self release.
	GDBMagmaClient aClientName: aclient. 
	^self initializeSession refresh.
! !

!GDBClientController class methodsFor: 'connection' stamp: 'lacm 11/14/2014 16:51'!
refresh: aClientName
	"This method permits to refresh a client session, where aclientName is the client name"
	self release.
      GDBMagmaClient aClientName: aClientName.
	^self initializeSession refresh.
! !

!GDBClientController class methodsFor: 'connection' stamp: 'lacm 11/14/2014 16:51'!
release
	GDBMagmaClient release.! !

!GDBClientController class methodsFor: 'removing' stamp: 'lacm 11/19/2014 21:28'!
remove: aGODData
	"This method permits to delete a GODData object in the datase"
	|myReader|
	(self database size = 0) 
		ifFalse: [
				myReader := self getAll. "database where:[:reader| reader read: #id from: 1]  distinct: true sortBy: #id descending: true."
    				myReader do: [:e| (e = aGODData ) 
									ifTrue: [GDBMagmaClient commit: [self database remove: e ].
										      Transcript show: aGODData ,' object was removed!!!!'; cr.	
											].
								].
				].
	myReader := self database where:[:reader| reader read: #id from: 1]  distinct: true sortBy: #id descending: true.
	myReader do: [:each| Transcript show: 'id: ',each id, '  author: ', each author, '  title: ', each title; cr.].! !

!GDBClientController class methodsFor: 'resetting' stamp: 'lacm 11/15/2014 18:31'!
resetAllDatabase
	"self createDatabase"
	self resetConferenceData.
	self resetData.
! !

!GDBClientController class methodsFor: 'resetting' stamp: 'lacm 11/14/2014 16:52'!
resetConferenceData
	self createConferenceCollection.
! !

!GDBClientController class methodsFor: 'resetting' stamp: 'lacm 11/14/2014 16:52'!
resetData
	self createGODDataCollection.
! !

!GDBClientController class methodsFor: 'adding' stamp: 'lacm 11/14/2014 16:52'!
saveConferences: aCollectionOfGCConference
	"This method permits to save a collection of GCConference objects in the datase"

	self createConferenceCollection.	
      GDBMagmaClient commit: [ 	self databaseConference add:  aCollectionOfGCConference.].	
		
	Transcript show: (aCollectionOfGCConference,' the collection of GCConference was saved!!!!'); cr; yourself.! !

!GDBClientController class methodsFor: 'searching' stamp: 'lacm 11/18/2014 18:25'!
searchAuthor: aString    
	"This method permits a search by author."
	
	| collection myReader temp|	
	
	temp:= '*',aString,'*'.
	collection:= self getAll.
	myReader:= collection select:[:each | temp match: each author].
	
	(myReader size = 0)	
		ifFalse:[Transcript show:' Author names with the word "', aString,'":'; cr.
				myReader do: [:each| Transcript show: 'Id: ',each id,'  Author: ', each author, '  Title: ', each title; cr.]
		];
		ifTrue:[	Transcript show:' The author ', aString,' does not exist in this database.'; cr.].
	
	^myReader! !

!GDBClientController class methodsFor: 'searching' stamp: 'lacm 11/19/2014 22:15'!
searchByAuthor: aString
	"This method searches GODData objects by Author and return a collection (i.e. an exact research)"
	|aBlock collection|
	(self getAll size > 0) 
	ifTrue:[
	aBlock := [:obj | obj author = aString].
	collection :=  GDBClientController searchFor: aBlock.
	collection do:[:e|Transcript show: 'Id: ', e id, 'Author: ', e author, 'Title: ', e title; cr.].
	^collection.
	].
^nil
	
! !

!GDBClientController class methodsFor: 'searching' stamp: 'lacm 11/19/2014 22:15'!
searchById: anInteger
	"This method searches GODData objects by Id and return an GODData object"
	|aBlock collection|
	(self getAll size > 0) 
	ifTrue:[
	aBlock := [:obj | obj id = anInteger].
	collection :=  GDBClientController searchFor: aBlock.
	collection do:[:e|Transcript show: 'Id: ', e id, 'Author: ', e author, 'Title: ', e title; cr. ^e.]].
^nil
	
! !

!GDBClientController class methodsFor: 'searching' stamp: 'lacm 11/19/2014 22:16'!
searchByTitle: aString
	"This method searches GODData objects by Title and return a collection (i.e. an exact research)"
	|aBlock collection|
	(self getAll size > 0) 
	ifTrue:[
	aBlock := [:obj | obj title = aString].
	collection :=  GDBClientController searchFor: aBlock.
	collection do:[:e|Transcript show: 'Id: ', e id, 'Author: ', e author, 'Title: ', e title; cr.].
	^collection.
	].
^nil! !

!GDBClientController class methodsFor: 'searching' stamp: 'lacm 11/14/2014 16:52'!
searchFor: aBlock    
	"This method searches for elements that satisfies aBlock conditions"
	| aBag collection|	

	collection :=  self getAll.

	aBag := Bag new.
	collection do: [:a| (aBlock value: a) ifTrue: [ aBag add: a.] ].
	^aBag.! !

!GDBClientController class methodsFor: 'searching' stamp: 'lacm 11/18/2014 18:28'!
searchTitle: aString    
	"This method permits a search by title."
	
	| collection myReader temp|	
	
	temp:= '*',aString,'*'.
	collection := self getAll .
	myReader:= collection select:[:each| temp match: each title].

	(myReader size = 0)	
		ifFalse:[Transcript show:' Titles that contain the word "', aString,'":'; cr.
				myReader do: [:each| Transcript show: 'Id: ',each id, 
												 '  Author: ', each author,
												 '  Title: ', each title; cr.].
		];
		ifTrue:[	Transcript show:' The title "', aString,'" not is in this database.'; cr.].
	
	^myReader! !

!GDBClientController class methodsFor: 'updating' stamp: 'lacm 11/19/2014 21:21'!
update: oldGODData with: newGODData
	"This method permits to update a GODData object in the database"
      |myReader auxReader|
      (oldGODData class = GODData) ifFalse: [GDBDataMustBeAnGODData signal].
      (newGODData class = GODData) ifFalse: [GDBDataMustBeAnGODData signal].
      (oldGODData author isNil) ifTrue:[GODDataAuthorMustContainsSomeInformation signal].
      (newGODData author isNil) ifTrue:[GODDataAuthorMustContainsSomeInformation signal].
      (oldGODData title isNil) ifTrue:[GODDataTitleMustContainsSomeInformation signal].
      (newGODData title isNil) ifTrue:[GODDataTitleMustContainsSomeInformation signal].
	(self database size =0)
		ifFalse:[
			auxReader := self getAll."self database where: [:reader | reader read: #id from:1] distinct:true sortBy:#id descending:true."
			auxReader do: [:e| (e = oldGODData)
								ifTrue:[  newGODData id: oldGODData id.
										GDBMagmaClient commit: [
												self database remove: e.
												self database add: newGODData.].	
										Transcript show: (oldGODData,' object was updated!!!!'); cr.			
									].
				].
			].
	myReader := self database where:[:reader| reader read: #id from: 1]  distinct: true sortBy: #id descending: true.		

	myReader do: [:each| Transcript show: 'id: ',each id, '  author: ', each author, '  title: ', each title; cr.].! !

!GDBClientController class methodsFor: 'private' stamp: 'lacm 11/19/2014 21:31'!
updateId: aGODData
	"this method updates the GODData id respect to the database"
	|newGODData newId myReader|	
	myReader := self database where: [:reader | reader read: #id from: 1].
	(myReader size = 0) 
		ifTrue: [newId :=1];
		ifFalse: [newId:= (myReader detectMax: [:each | each id]) id +1].
	aGODData id: newId.
	newGODData:= aGODData copy.
	^newGODData! !

Object subclass: #GDBMagmaClient
	instanceVariableNames: 'session'
	classVariableNames: 'DefaultConnection'
	poolDictionaries: ''
	category: 'GODBases'!

GDBMagmaClient class
	instanceVariableNames: 'aClientName'!

!GDBMagmaClient commentStamp: 'lacm 11/8/2014 13:19' prior: 0!
A GDBMagmaClient is a class that permits to initialize a client session.!

GDBMagmaClient class
	instanceVariableNames: 'aClientName'!

!GDBMagmaClient class methodsFor: 'private' stamp: 'lacm 11/8/2014 12:38'!
aClientName
	^aClientName! !

!GDBMagmaClient class methodsFor: 'private' stamp: 'lacm 11/8/2014 12:38'!
aClientName: aString
	aClientName:= aString! !

!GDBMagmaClient class methodsFor: 'connection' stamp: 'lacm 9/28/2014 20:53'!
commit: aBlock
	self default session commit: aBlock! !

!GDBMagmaClient class methodsFor: 'connection' stamp: 'lacm 11/8/2014 12:36'!
default
	^ DefaultConnection ifNil: [
			DefaultConnection := self new connect: aClientName; 
			yourself
			].! !

!GDBMagmaClient class methodsFor: 'initialize-release' stamp: 'lacm 11/8/2014 12:37'!
initialize
	aClientName:= 'lucy_'! !

!GDBMagmaClient class methodsFor: 'initialize-release' stamp: 'lacm 10/15/2014 12:21'!
release
	(DefaultConnection isNil)
	ifFalse:[
			DefaultConnection session disconnectAndClose.
			DefaultConnection := nil.
		]! !

!GDBMagmaClient class methodsFor: 'connection' stamp: 'lacm 11/1/2014 01:24'!
root
"This method retrieves the connection for access to the session, and then access to the root method of Magma "
	^ self session root! !

!GDBMagmaClient class methodsFor: 'connection' stamp: 'lacm 11/1/2014 01:24'!
session
	^ self default session! !

!GDBMagmaClient methodsFor: 'connection' stamp: 'lacm 11/8/2014 12:27'!
connect
	session := (MagmaRemoteLocation
					host: 'localhost'
					port: 51001) newSession.
	session connectAs: 'lucy'.! !

!GDBMagmaClient methodsFor: 'connection' stamp: 'lacm 11/8/2014 12:27'!
connect: aString
	session := (MagmaRemoteLocation
					host: 'localhost'
					port: 51001) newSession.
	session connectAs: aString.! !

!GDBMagmaClient methodsFor: 'private' stamp: 'lacm 9/28/2014 20:51'!
session
	^ session! !

!GDBMagmaClient methodsFor: 'private' stamp: 'lacm 10/15/2014 12:23'!
session: anObject
	session := anObject! !

Object subclass: #GDBMagmaServer
	instanceVariableNames: 'gdbServer repositoryName'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GODBases'!

!GDBMagmaServer commentStamp: 'lacm 11/12/2014 15:09' prior: 0!
A GDBMagmaServer is a class that permits start and stop the server.

GDBServerController startServer!

!GDBMagmaServer methodsFor: 'repository' stamp: 'lacm 11/3/2014 16:05'!
createGDBRepository
self isCreated
      ifFalse:
	[MagmaRepositoryController
		create: self repositoryName
		root:  Dictionary new].! !

!GDBMagmaServer methodsFor: 'repository' stamp: 'lacm 11/12/2014 14:43'!
deleteGDBRepository
	self isCreated      
		ifTrue:	[GDBClientController release.	
				 self stopGDBServer.	
				 MagmaRepositoryController		
						delete: self repositoryName.].! !

!GDBMagmaServer methodsFor: 'repository' stamp: 'lacm 11/13/2014 21:39'!
detectRunningMagmaServerConsole
	^MagmaServerConsole allInstances
		detect: [:aServerConsole| aServerConsole isRunning]
		ifNone: [^False "self error: 'No db running!!'"]! !

!GDBMagmaServer methodsFor: 'initialize-release' stamp: 'lacm 11/11/2014 20:45'!
initialize	
	repositoryName:= 'repositoryExample1'! !

!GDBMagmaServer methodsFor: 'repository' stamp: 'lacm 11/13/2014 21:23'!
inspectServer
	self detectRunningMagmaServerConsole inspect! !

!GDBMagmaServer methodsFor: 'repository' stamp: 'lacm 11/3/2014 16:03'!
isCreated	
	^(FileDirectory forFileName: '') directoryExists: self repositoryName! !

!GDBMagmaServer methodsFor: 'private' stamp: 'lacm 10/15/2014 12:09'!
port
	^ 51001! !

!GDBMagmaServer methodsFor: 'private' stamp: 'lacm 11/8/2014 11:23'!
repositoryName
	^  repositoryName! !

!GDBMagmaServer methodsFor: 'private' stamp: 'lacm 11/8/2014 11:23'!
repositoryName: aRepositoryName
	 repositoryName:= aRepositoryName.! !

!GDBMagmaServer methodsFor: 'connection' stamp: 'lacm 11/3/2014 16:45'!
startGDBServer
	self createGDBRepository.
	self stopGDBServer.
	gdbServer := MagmaServerConsole new
		open: self repositoryName;
		processOn: self port;
		yourself.
	self inspect! !

!GDBMagmaServer methodsFor: 'connection' stamp: 'lacm 11/5/2014 15:11'!
stopGDBServer
	MagmaServerConsole allInstancesDo:
		[:aServerConsole| aServerConsole shutdown]! !

Object subclass: #GDBServerController
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GODBases'!

GDBServerController class
	instanceVariableNames: 'uniqueServerInstance'!

!GDBServerController commentStamp: 'lacm 11/15/2014 21:10' prior: 0!
A GDBControllerServer is a main class that permits differents transactions in the server.

To create the repository and connect the server, run only once the following line in a workspace:

		GDBServerController startServer

Then, save it as an image server. To close the server, close that image saved, and when you open it again, the server will already be running.

To delete the repository run the following:

		GDBServerController deleteRepository

this command stops the server.
		
To know if the server was started run the following:

		GDBServerController isStarted!

GDBServerController class
	instanceVariableNames: 'uniqueServerInstance'!

!GDBServerController class methodsFor: 'repository' stamp: 'lacm 11/12/2014 11:11'!
createRepository	
	"This method permits to create a repository (i.e., the database)"
	self uniqueServerInstance createGDBRepository.! !

!GDBServerController class methodsFor: 'repository' stamp: 'lacm 11/11/2014 21:11'!
deleteRepository	
	"This method permits to delete the repository (i.e., the database)"
	self uniqueServerInstance deleteGDBRepository.! !

!GDBServerController class methodsFor: 'connection' stamp: 'lacm 11/13/2014 21:43'!
isStarted
	(self uniqueServerInstance detectRunningMagmaServerConsole = False) ifTrue:[^False].
	^True! !

!GDBServerController class methodsFor: 'connection' stamp: 'lacm 11/11/2014 21:08'!
startServer
	"This method permits to start the server"
	self uniqueServerInstance startGDBServer.! !

!GDBServerController class methodsFor: 'connection' stamp: 'lacm 11/12/2014 11:08'!
stopServer
	"This method permits to stop the server"
	self uniqueServerInstance stopGDBServer.
	uniqueServerInstance := nil.! !

!GDBServerController class methodsFor: 'accessing' stamp: 'lacm 11/11/2014 21:06'!
uniqueServerInstance
	"This method permits to start the server"
	uniqueServerInstance ifNil: [uniqueServerInstance:= GDBMagmaServer new].
	^uniqueServerInstance! !
GDBMagmaClient initialize!
